rem Author:  Mark Lang, 1998
rem Name:    desc.sql
rem Purpose: Enhanced sql*plus-like describe for tables
rem Usage:   @desc <%owner.table%> <keys>
rem Subject: object:table
rem Attrib:  pls v73
rem Descr:   Like sql*plus describe but includes key/index information
rem          P1(2)
rem          || + column position in key
rem          |+ nth key of same type (for reference)
rem          + type of key P=primary U=unique F=foreign B=bitmap I=index
rem Notes:   keys=<PURIB|%|->
rem          Supports only tables and views; can specify wildcards
rem          You cannot distiguish unique indexes from unique constraints
rem          enh: Should add refs from other tables?
rem          enh: Support procs / pkgs too
rem SeeAlso: @coldefs @colstat
rem History:
rem          01-feb-98  Initial release
rem          12-jan-98  Created

@setup1
set serveroutput on size 10240
define keys="upper(replace('&&2','%','PURIB'))"

declare

procedure describe(town varchar2, tnam varchar2)
is
  c1 number := 27;
  c2 number := 8;
  c3 number := 15;
  c4 number := 25;

  type keyrec_t is record (
    nam varchar2(30)
  , typ char(1)
  , pos number
  );
  type keytab_t is table of keyrec_t index by binary_integer;
  type colrec_t is record (
    nam varchar2(30)
  , pos number
  , key binary_integer
  );
  type coltab_t is table of colrec_t index by binary_integer;
  keytab keytab_t;
  coltab coltab_t;
  keynum binary_integer := 0;
  colnum binary_integer := 0;

  pnum binary_integer := 0;
  unum binary_integer := 0;
  rnum binary_integer := 0;
  inum binary_integer := 0;
  bnum binary_integer := 0;

  i binary_integer;
  n binary_integer;
  c varchar2(2);

  txt varchar2(512);

  procedure get_con_cols
  is
  begin
      -- BUILD KEY INFORMATION TABLES
    for r in (
      select *
      from &&ora._constraints
      where owner = town
      and table_name = tnam
      and constraint_type in ('P', 'U', 'R')
      order by decode(constraint_type, 'P', 1, 'U', 2, 'R', 3, 4)
    ) loop

      keynum := keynum + 1;
      keytab(keynum).nam := r.constraint_name;
      if    r.constraint_type = 'P' then pnum := pnum + 1; n := pnum; c := 'P';
      elsif r.constraint_type = 'U' then unum := unum + 1; n := unum; c := 'U';
      else                               rnum := rnum + 1; n := rnum; c := 'R';
      end if;
      keytab(keynum).typ := c;
      keytab(keynum).pos := n;

      for s in (
        select *
        from &&ora._cons_columns
        where owner = r.owner
        and constraint_name = r.constraint_name
        order by position
      ) loop
        colnum := colnum + 1;
        coltab(colnum).key := keynum;
        coltab(colnum).nam := s.column_name;
        coltab(colnum).pos := s.position;
      end loop;
    end loop;
  end;

  procedure get_ind_cols
  is
  begin
    for r in (
      select *
      from &&ora._indexes i
      where table_owner = town
      and table_name = tnam
      and not exists (
        select 0
        from all_constraints
        where owner = i.table_owner
        and constraint_name = i.index_name
      )
      order by decode(uniqueness, 'UNIQUE', 1, 'BITMAP', 3, 2)
    ) loop

      keynum := keynum + 1;
      keytab(keynum).nam := r.index_name;
      if    r.uniqueness = 'UNIQUE' then unum := unum + 1; n := unum; c := 'U';
      elsif r.uniqueness = 'BITMAP' then bnum := bnum + 1; n := bnum; c := 'B';
      else                               inum := inum + 1; n := inum; c := 'I';
      end if;
      keytab(keynum).typ := c;
      keytab(keynum).pos := n;

      for s in (
        select *
        from &&ora._ind_columns
        where index_owner = r.owner
        and index_name = r.index_name
        order by column_position
      ) loop
        colnum := colnum + 1;
        coltab(colnum).key := keynum;
        coltab(colnum).nam := s.column_name;
        coltab(colnum).pos := s.column_position;
      end loop;
    end loop;
  end;

begin
-- COLUMN_NAME    NULL?    TYPE    KEYS
--                                 P,U,F,I,B,R

  if &&keys != '-' and nvl(length(&&keys), 0) > 0 then
    get_con_cols;
    get_ind_cols;
  end if;

  -- OUTPUT DESCRIBE
  dbms_output.put_line(
    rpad('COLUMN_NAME', c1)||' '
  ||rpad('NULL?', c2)||' '
  ||rpad('TYPE', c3)||' '
  ||rpad('KEYS', c4)
  );
  dbms_output.put_line(
    rpad('-', c1, '-')||' '
  ||rpad('-', c2, '-')||' '
  ||rpad('-', c3, '-')||' '
  ||rpad('-', c4, '-')
  );

  for r in (
    select
      column_name d_name
    , decode(nullable, 'N', 'NOT NULL', ' ') d_null
    , data_type
      ||decode(decode(data_type, 'FLOAT', 'NUMBER', data_type)
        , 'DATE', ''
        , 'NUMBER', decode(data_precision, null, '', '('||to_char(data_precision)
          ||decode(data_scale, null, '', 0, '', ','||to_char(data_scale))||')')
        , decode(data_length, null, '', 0, '', '('||to_char(data_length)||')')
      )
      d_type
    from all_tab_columns
    where owner = town
    and table_name = tnam
  ) loop
    txt := rpad(r.d_name, c1)||' '
      ||rpad(r.d_null, c2)||' '
      ||rpad(r.d_type, c3)||' ';
    for i in 1..colnum loop
      if coltab(i).nam = r.d_name then
        n := coltab(i).key;
        if instr(&&keys, keytab(n).typ) > 0 then
          txt := txt||rpad(keytab(n).typ||to_char(keytab(n).pos)
            ||'('||to_char(coltab(i).pos)||')',6);
        end if;
      end if;
    end loop;
    dbms_output.put_line(txt);
  end loop;
end;

begin
  for r in (
    select *
    from &&ora._objects
    where owner like &&o1
    and object_name like &&n1
    and object_type in ('TABLE', 'VIEW')
    order by owner, object_name
  ) loop
    dbms_output.put_line('('||r.owner||'.'||r.object_name||' '||r.object_type||')');
    describe(r.owner, r.object_name);
    dbms_output.put_line(chr(10));
  end loop;
end;
/

undef keys

@setdefs
